#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""飞书机器人和消息会话事件

https://open.feishu.cn/document/ukTMukTMukTM/uMTNxYjLzUTM24yM1EjN
"""
from enum import Enum
from typing import List, Optional

from .base import BaseModel, EventType, Operator, User, EventContent


class ChatI18Names(BaseModel):
    en_us: str = ""
    zh_cn: str = ""


class BotEvent(EventContent):
    app_id: str
    chat_i18_names: Optional[ChatI18Names] = None
    chat_name: str
    chat_owner_employee_id: str
    chat_owner_open_id: str
    open_chat_id: str
    operator_employee_id: str
    operator_name: str
    operator_open_id: str
    owner_is_bot: bool
    tenant_key: str
    type: EventType


class AddBotEvent(BotEvent):
    """机器人进群

    {
        "app_id": "cli_9c8609450f78d102",
        "chat_i18n_names": { // 群名称国际化字段
            "en_us": "英文标题",
            "zh_cn": "中文标题"
        },
        "chat_name": "群名称",
        "chat_owner_employee_id": "ca51d83b",// 群主的employee_id（即“用户ID”。如果群主是机器人则没有这个字段，仅企业自建应用返回）
        "chat_owner_name": "xxx", // 群主姓名
        "chat_owner_open_id": "ou_18eac85d35a26f989317ad4f02e8bbbb", // 群主的open_id
        "open_chat_id": "oc_e520983d3e4f5ec7306069bffe672aa3",  // 群聊的id
        "operator_employee_id": "ca51d83b", // 操作者的emplolyee_id ，仅企业自建应用返回
        "operator_name": "yyy", // 操作者姓名
        "operator_open_id": "ou_18eac85d35a26f989317ad4f02e8bbbb",//操作者的open_id
        "owner_is_bot": false, //群主是否是机器人
        "tenant_key": "736588c9260f175d",  // 企业标识
        "type": "add_bot" // 事件类型
    }
    """

    type: EventType = EventType.ADD_BOT


class RemoveBotEvent(BaseModel):
    """机器人被移出群

    {
        "app_id": "cli_9c8609450f78d102",
        "chat_i18n_names": { // 群名称国际化字段
            "en_us": "英文标题",
            "zh_cn": "中文标题"
        },
        "chat_name": "群名称",
        "chat_owner_employee_id": "ca51d83b",// 群主的employee_id（即“用户ID”。如果群主是机器人则没有这个字段，仅企业自建应用返回）
        "chat_owner_name": "xxx", // 群主姓名
        "chat_owner_open_id": "ou_18eac85d35a26f989317ad4f02e8bbbb", // 群主的open_id
        "open_chat_id": "oc_e520983d3e4f5ec7306069bffe672aa3",  // 群聊的id
        "operator_employee_id": "ca51d83b", // 操作者的emplolyee_id ，仅企业自建应用返回
        "operator_name": "yyy", // 操作者姓名
        "operator_open_id": "ou_18eac85d35a26f989317ad4f02e8bbbb",//操作者的open_id
        "owner_is_bot": false, //群主是否是机器人
        "tenant_key": "736588c9260f175d",  // 企业标识
        "type": "remove_bot" // 移除机器人：remove_bot
    }
    """

    type: EventType = EventType.REMOVE_BOT


class P2PChatCreateEvent(EventContent):
    """用户和机器人的会话首次被创建

    {
        "app_id": "cli_9c8609450f78d102",
        "chat_id": "oc_26b66a5eb603162b849f91bcd8815b20", //机器人和用户的会话id
        "operator": { // 会话的发起人。可能是用户，也可能是机器人。
            "open_id": "ou_2d2c0399b53d06fd195bb393cd1e38f2" // 员工对此应用的唯一标识，同一员工对不同应用的open_id不同
            "user_id": "gfa21d92"  // 即“用户ID”，仅企业自建应用会返回
        },
        "tenant_key": "736588c9260f175c",  // 企业标识
        "type": "p2p_chat_create",  // 事件类型
        "user": {  // 会话的用户
            "name": "张三",
            "open_id": "ou_7dede290d6a27698b969a7fd70ca53da",  // 员工对此应用的唯一标识，同一员工对不同应用的open_id不同
            "user_id": "gfa21d92" // 即“用户ID”，仅企业自建应用会返回
        }
    }
    """

    app_id: str
    chat_id: str
    operator: Operator
    tenant_key: str
    type: EventType = EventType.P2P_CHAT_CREATE
    user: User


class EventMsgType(str, Enum):
    TEXT = "text"
    POST = "post"
    IMAGE = "image"
    FILE = "file"
    MERGE_FORWARD = "merge_forward"


class EventChatType(str, Enum):
    PRIVATE = "private"
    GROUP = "group"


class MessageEvent(EventContent):
    """接收消息

    {
        "type": "message", // 事件类型
        "app_id": "cli_xxx",
        "tenant_key": "xxx", //企业标识
        "root_id": "",
        "parent_id": "",
        "open_chat_id": "oc_5ce6d572455d361153b7cb51da133945",
        "chat_type": "private",//私聊private，群聊group
        "msg_type": "text",    //消息类型
        "open_id": "ou_18eac85d35a26f989317ad4f02e8bbbb",
        "open_message_id": "om_36686ee62209da697d8775375d0c8e88",
        "is_mention": false,
        "text": "<at open_id="xxx">@小助手</at> 消息内容 <at open_id="yyy">@张三</at>",      // 消息文本，可能包含被@的人/机器人。
        "text_without_at_bot":"消息内容 <at open_id="yyy">@张三</at>" //消息内容，会过滤掉at你的机器人的内容
    }
    """

    type: EventType = EventType.MESSAGE
    app_id: str
    tenant_key: str
    root_id: str = ""
    parent_id: str = ""
    open_chat_id: str
    chat_type: EventChatType
    msg_type: EventMsgType
    open_id: str
    open_message_id: str
    is_mention: bool


class TextMessageEvent(MessageEvent):
    """文本消息

    {
        "type": "message", // 事件类型
        "app_id": "cli_xxx",
        "tenant_key": "xxx", //企业标识
        "root_id": "",
        "parent_id": "",
        "open_chat_id": "oc_5ce6d572455d361153b7cb51da133945",
        "chat_type": "private",//私聊private，群聊group
        "msg_type": "text",    //消息类型
        "open_id": "ou_18eac85d35a26f989317ad4f02e8bbbb",
        "open_message_id": "om_36686ee62209da697d8775375d0c8e88",
        "is_mention": false,
        "text": "<at open_id="xxx">@小助手</at> 消息内容 <at open_id="yyy">@张三</at>",      // 消息文本，可能包含被@的人/机器人。
        "text_without_at_bot":"消息内容 <at open_id="yyy">@张三</at>" //消息内容，会过滤掉at你的机器人的内容
    }
    """

    text: str
    text_without_at_bot: str


class PostMessageEvent(MessageEvent):
    """富文本和 post 消息

    {
        "type": "message",
        "app_id": "cli_xxx",
        "tenant_key": "xxx", //企业标识
        "root_id": "",
        "parent_id": "",
        "open_chat_id": "oc_5ce6d572455d361153b7cb51da133945",//发消息的open_chat_id
        "chat_type": "private",
        "msg_type": "post",// rich_text和post消息这里统一都是post
        "open_id": "ou_18eac85d35a26f989317ad4f02e8bbbb",//发消息的用户open_id
        "open_message_id": "om_a81fa00ee467b1084ffd80b197b58f4b",//消息id
        "is_mention": false,
        "text": "\u003cp\u003e测试1212\u003c/p\u003e\u003cfigure\u003e\u003cimg src=\"http://p4.pstatp.com/origin/daff000d4967d033705b\" origin-width=\"2456\" origin-height=\"2458\"/\u003e\u003c/figure\u003e",//消息内容
        "text_without_at_bot":"消息内容",//消息内容，会过滤掉at你的机器人的内容
        "title": "测试" ,//消息标题
        "image_keys": [ //富文本里面的图片的keys
           "1867eac8-8006-40be-8549-b7beae0d3c4a",
           "434593af-5269-4db4-8b94-65c6dfd4f35e"
         ],
    }
    """

    text: str
    text_without_at_bot: str
    msg_type: EventMsgType = EventMsgType.POST
    title: str
    image_keys: List[str]


class ImageMessageEvent(MessageEvent):
    """图片image消息

    {
        "type": "message",
        "app_id": "cli_xxx",
        "tenant_key": "xxx", //企业标识
        "root_id": "",
        "parent_id": "",
        "open_chat_id": "oc_5ce6d572455d361153b7cb51da133945",
        "chat_type": "private",
        "msg_type": "image", //图片消息
        "image_height" :"300",
        "image_width" :"300",
        "open_id": "ou_18eac85d35a26f989317ad4f02e8bbbb",
        "open_message_id": "om_340057d660022bf141eb470859c6114c",
        "is_mention": false,
        "image_key": "cd1ce282-94d1-4154-a326-121b07e4721e", // image_key，获取图片内容请查https://open.feishu.cn/document/ukTMukTMukTM/uYzN5QjL2cTO04iN3kDN
    }
    """

    msg_type: EventMsgType = EventMsgType.IMAGE
    image_height: str
    image_width: str
    image_key: str


class FileMessageEvent(MessageEvent):
    """文件file消息

    {
        "app_id": "cli_xxx",
        "tenant_key": "xxx",
        "chat_type": "private",
        "is_mention": false,
        "msg_type": "file",
        "open_chat_id": "oc_a1e061372f7745a543dsd5h3d6d2349a",
        "open_id": "ou_2b940d169b7a4a0c76633984b08ced73",
        "open_message_id": "om_d85c4sd7b209tbb98g693a958bc7185f",
        "parent_id": "",
        "root_id": "",
        "type": "message",
        "file_key": "file_b4do9r9b-3526-4bc4-a568-65fe3695b05g"
    }
    """

    msg_type: EventMsgType = EventMsgType.FILE
    file_key: str


class Msg(BaseModel):
    """
    {
        "root_id": "",
        "parent_id": "",
        "open_chat_id": "oc_b74c59c68d0f2d0ac65846272499d651",
        "msg_type": "post",
        "open_id": "",
        "open_message_id": "om_5d5b1732aa9b997dff23d63146427bb2",
        "is_mention": false,
        "text": "\u003cp\u003e富文本内容\u003c/p\u003e\u003cfigure\u003e\u003cimg src=\"http://p2.pstatp.com/origin/dad90010d9c8fc72f0b0\" origin-width=\"888\" origin-height=\"888\"/\u003e\u003c/figure\u003e",
        "title": "富文本",
        "create_time": 1550044772
    }
    """

    root_id: str = ""
    parent_id: str = ""
    open_chat_id: str = ""
    msg_type: EventMsgType = EventMsgType.TEXT
    open_id: str = ""
    open_message_id: str = ""
    is_mention: bool = False
    text: str = ""
    user_open_id: str = ""
    image_key: str = ""
    image_url: str = ""
    file_key: str = ""
    title: str = ""
    create_time: int = 0


class MergeForwardMessageEvent(MessageEvent):
    """合并转发消息内容

    {
        "type": "message",
        "app_id": "cli_xxx",
        "tenant_key": "xxx", //企业标识
        "root_id": "",
        "parent_id": "",
        "open_chat_id": "oc_5ce6d572455d361153b7cb51da133945",
        "msg_type": "merge_forward",
        "open_id": "ou_18eac85d35a26f989317ad4f02e8bbbb",
        "open_message_id": "om_b3961b120d67259e7495d8eb69488189",
        "is_mention": false,
        "chat_type": "private",
        "chat_id": "6642174187597201422",
        "user": "6610187460791558158",
        "msg_list": [
        {
            "root_id": "",
            "parent_id": "",
            "open_chat_id": "oc_b74c59c68d0f2d0ac65846272499d651",
            "msg_type": "image",
            "open_id": "",
            "open_message_id": "be1000265b014075a869134b20c87633",
            "is_mention": false,
            "image_key": "99295878-5e85-41a3-bb00-0ad63b5b156d",
            "image_url": "https://oapi-staging.zjurl.cn/open-apis/api/v2/file/f/99295878-5e85-41a3-bb00-0ad63b5b156d",
            "create_time": 1550044148
       },
       {
            "root_id": "",
            "parent_id": "",
            "open_chat_id": "oc_b74c59c68d0f2d0ac65846272499d651",
            "msg_type": "text",
            "open_id": "",
            "open_message_id": "om_a96c620f2aa036e3c08abebaec7f09d1",
            "is_mention": false,
            "text": "文本1",
            "create_time": 1550044749
       },
       {
            "root_id": "",
            "parent_id": "",
            "open_chat_id": "oc_b74c59c68d0f2d0ac65846272499d651",
            "msg_type": "post",
            "open_id": "",
            "open_message_id": "om_5d5b1732aa9b997dff23d63146427bb2",
            "is_mention": false,
            "text": "\u003cp\u003e富文本内容\u003c/p\u003e\u003cfigure\u003e\u003cimg src=\"http://p2.pstatp.com/origin/dad90010d9c8fc72f0b0\" origin-width=\"888\" origin-height=\"888\"/\u003e\u003c/figure\u003e",
            "title": "富文本",
            "create_time": 1550044772
       }
       ]
    }
    """

    user: str = ""
    msg_list: List[Msg]


class MessageReadEvent(EventContent):
    """消息已读

    {
        "app_id": "cli_9c8609450f78d102",
        "open_chat_id": "oc_e520983d3e4f5ec7306069bffe672aa3",
        "open_id": "ou_2d2c0399b53d06fd195bb393cd1e38f2",
        "open_message_ids": ["om_dc13264520392913993dd051dba21dcf"],   // 已读消息列表
        "tenant_key": "xxx",
        "type": "message_read"
    }
    """

    app_id: str
    open_chat_id: str
    open_id: str
    open_message_ids: List[str]
    tenant_key: str
    type: EventType = EventType.MESSAGE_READ



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""订阅事件处理

包含订阅事件的flask/sanic blueprint实现

https://open.feishu.cn/document/ukTMukTMukTM/uUTNz4SN1MjL1UzM
"""
import asyncio
import logging
from concurrent.futures import Executor
from concurrent.futures import ThreadPoolExecutor
from typing import Optional, Callable, Union, Awaitable

from pydantic import ValidationError

from .base import decrypt_aes
from ..errors import ERRORS, FeishuError
from ..models.events import *

logger = logging.getLogger("feishu")


def setup_event_blueprint(framework: str, blueprint: Union["flask.Blueprint", "sanic.Blueprint"],
                          path: str, on_event: callable, executor: Optional[Executor] = None,
                          verify_token: Optional[str] = None, encrypt_key: Optional[str] = None):
    """配置一个用于接收订阅事件的Blueprint

    https://open.feishu.cn/document/ukTMukTMukTM/uUTNz4SN1MjL1UzM
    Args:
        framework: 可选"flask"/"sanic"
        blueprint: flask/sanic的Blueprint对象
        path: 回调路径, 只需包含blueprint后的挂载部分
        on_event:
            有订阅事件时, 接收Event的参数, 无需返回
            当framework="flask"时,
                on_event函数会放在executor线程中单独调用
                注意，为了不卡住线程池，请自行处理完出错信息，否则出错Event会被丢弃
            当framework="sanic"时, on_event函数会被asyncio.create_task添加，
                请确保asyncio的loop配置正确，e.g. set_default_loop, 或者注意运行的先后顺序
                一般情况下直接用sanic不应该出现任何问题
        executor: 用来后台执行event的executor，没有的话默认启动一个2线程的
        verify_token: 校验token, 需和飞书后台配置一致, 不提供则不校验请求来源
        encrypt_key: 加密key, 需和飞书后台配置一致, 不提供则无法解析加密数据
    """
    if framework == "flask":
        return flask_blueprint(blueprint=blueprint, path=path, on_event=on_event, executor=executor,
                               verify_token=verify_token, encrypt_key=encrypt_key)
    elif framework == "sanic":
        return sanic_blueprint(blueprint=blueprint, path=path, on_event=on_event,
                               verify_token=verify_token, encrypt_key=encrypt_key)
    else:
        raise NotImplementedError


def flask_blueprint(blueprint: "flask.Blueprint", path: str,
                    on_event: Callable[[Event], None], executor: Optional[Executor] = None,
                    verify_token: Optional[str] = None, encrypt_key: Optional[str] = None):
    """配置一个用于接收消息交互回调的blueprint

    Args:
        blueprint: flask的Blueprint对象
        path: 回调路径, 只需包含blueprint后的挂载部分
        on_event: 有订阅事件时, 接收Event的参数, 无需返回,
            注意，因为on_event会在executor中实现，请自行处理完出错信息，否则出错Event会被丢弃
        executor: 用来后台执行event的executor，没有的话默认启动一个2线程的
        verify_token: 校验token, 需和飞书后台配置一致, 不提供则不校验请求来源
        encrypt_key: 加密key, 需和飞书后台配置一致, 不提供则无法解析加密数据
    """
    import flask

    if not executor:
        executor = ThreadPoolExecutor(2)

    def on_event_wrapper(event: Event):
        try:
            on_event(event)
        except Exception as e:
            logger.exception(f"执行on_event时失败, event={event}")

    @blueprint.route(path, methods=["POST"])
    def handle_event():
        payload: dict = flask.request.get_json(force=True)
        if "encrypt" in payload:
            payload = decrypt_aes(encrypt_key, payload["encrypt"])

        payload_type = payload.get("type")
        if payload_type == "url_verification":
            return url_verification(payload)
        elif payload_type == "event_callback":
            event = Event(**payload)
            event.event = guess_event(payload.get("event"))
            executor.submit(on_event_wrapper, event)
        return flask.jsonify()

    def url_verification(payload: dict):
        """配置请求网址后飞书会发送的验证请求

        https://open.feishu.cn/document/ukTMukTMukTM/uUTNz4SN1MjL1UzM#%E9%85%8D%E7%BD%AE%E8%AF%B7%E6%B1%82%E7%BD%91%E5%9D%80
        """
        if verify_token and verify_token != payload.get("token"):
            return flask.jsonify(challenge="")

        return flask.jsonify(challenge=payload.get("challenge"))


def sanic_blueprint(blueprint: "sanic.Blueprint", path: str,
                    on_event: Callable[[Event], Awaitable[None]],
                    verify_token: Optional[str] = None, encrypt_key: Optional[str] = None):
    """配置一个用于接收消息交互回调的sanic.blueprint

    Args:
        blueprint: sanic的Blueprint对象
        path: 回调路径, 只需包含blueprint后的挂载部分
        on_event: 有Action事件时, 接收Event类型的参数, 无需任何返回
            on_event函数会被asyncio.create_task添加，
            请确保asyncio的loop配置正确，e.g. set_default_loop, 或者注意运行的先后顺序
            一般情况下直接用sanic不应该出现任何问题
        verify_token: 校验token, 需和飞书后台配置一致, 不提供则不校验请求来源
        encrypt_key: 加密key, 需和飞书后台配置一致, 不提供则无法解析加密数据
    """
    from sanic.request import Request
    from sanic import response

    @blueprint.route(path, methods=["POST"])
    async def handle_event(request: Request):
        payload: dict = request.json
        if "encrypt" in payload:
            payload = decrypt_aes(encrypt_key, payload["encrypt"])

        payload_type = payload.get("type")
        if payload_type == "url_verification":
            return url_verification(payload)
        elif payload_type == "event_callback":
            event = Event(**payload)
            event.event = guess_event(payload.get("event"))
            asyncio.create_task(on_event(event))

        return response.json({})

    def url_verification(payload: dict):
        """配置请求网址后飞书会发送的验证请求

        https://open.feishu.cn/document/ukTMukTMukTM/uUTNz4SN1MjL1UzM#%E9%85%8D%E7%BD%AE%E8%AF%B7%E6%B1%82%E7%BD%91%E5%9D%80
        """
        if verify_token and verify_token != payload.get("token"):
            return response.json(dict(challenge=""))

        return response.json(dict(challenge=payload.get("challenge")))


def guess_event(event: dict) -> Union[dict, EventContent]:
    """适配event的真实类型"""
    event_type = event.get("type")
    # print("event_type", event_type)
    msg_event_cls: Optional[EventContent] = None
    if event_type == EventType.MESSAGE:
        msg_type = event.get("msg_type")
        # print("msg_type", msg_type)
        msg_event_cls = {
            EventMsgType.TEXT: TextMessageEvent,
            EventMsgType.IMAGE: ImageMessageEvent,
            EventMsgType.POST: PostMessageEvent,
            EventMsgType.FILE: FileMessageEvent,
            EventMsgType.MERGE_FORWARD: MergeForwardMessageEvent
        }.get(msg_type)
        # print("msg_event_cls", msg_event_cls)

    event_cls: Optional[EventContent] = {
        # app
        EventType.APP_OPEN: AppOpenEvent,
        EventType.APP_STATUS_CHANGE: AppStatusChangeEvent,
        EventType.APP_TICKET: AppTicketEvent,
        EventType.APP_UNINSTALLED: AppUninstalledEvent,

        # contact
        EventType.USER_ADD: UserAddEvent,
        EventType.DEPT_ADD: DeptAddEvent,
        EventType.USER_STATUS_CHANGE: UserStatusChangeEvent,
        EventType.CONTACT_SCOPE_CHANGE: ContactScopeChangeEvent,

        # bot
        EventType.ADD_BOT: AddBotEvent,
        EventType.REMOVE_BOT: RemoveBotEvent,
        EventType.P2P_CHAT_CREATE: P2PChatCreateEvent,
        EventType.MESSAGE_READ: MessageReadEvent,
        EventType.MESSAGE: msg_event_cls,

        # chat
        EventType.ADD_USER_TO_CHAT: UserChatEvent,
        EventType.REMOVE_USER_FROM_CHAT: UserChatEvent,
        EventType.REVOKE_ADD_USER_FROM_CHAT: UserChatEvent,
        EventType.CHAT_DISBAND: ChatDisbandEvent,

        # approval
        EventType.LEAVE_APPROVAL: LeaveApprovalEvent,
        EventType.LEAVE_APPROVAL_V2: LeaveApprovalV2Event,
        EventType.WORK_APPROVAL: WorkApprovalEvent,
        EventType.OUT_APPROVAL: OutApprovalEvent,
        EventType.TRIP_APPROVAL: TripApprovalEvent,
        EventType.SHIFT_APPROVAL: ShiftApprovalEvent,
        EventType.REMEDY_APPROVAL: RemedyApprovalEvent,

        # calendar
        EventType.EVENT_REPLY: EventReplyEvent,
    }.get(event_type)

    if event_cls:
        try:
            return event_cls(**event)
        except ValidationError:
            raise FeishuError(ERRORS.VALIDATION_ERROR, f"解析事件失败，原始数据 event = {event}")
    else:
        # 不知道是啥类型，直接返回原始的dict
        return event



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .apis import setup_action_blueprint, setup_event_blueprint, guess_event
from .client import FeishuClient
from .errors import FeishuError, ERRORS
from .models import *
from .stores import TokenStore, MemoryStore, RedisStore
from .version import __version__



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""飞书订阅事件

参考订阅事件文档 https://open.feishu.cn/document/ukTMukTMukTM/uUTNz4SN1MjL1UzM
"""
from .app import (AppOpenEvent, AppStatusChangeEvent, OrderPaidEvent, AppTicketEvent, AppUninstalledEvent, Applicant,
                  Installer)
from .approval import (LeaveApprovalEvent, LeaveApprovalV2Event, WorkApprovalEvent, ShiftApprovalEvent,
                       TripApprovalEvent, OutApprovalEvent, RemedyApprovalEvent, TimeUnit, Schedule, I18nResource)
from .base import (EventType, User, Operator, AppStatus, BuyType, UserChatEventType, PricePlanType, Event, EventContent)
from .bot import (TextMessageEvent, ImageMessageEvent, PostMessageEvent, FileMessageEvent, MergeForwardMessageEvent,
                  AddBotEvent, RemoveBotEvent, P2PChatCreateEvent, MessageReadEvent, ChatI18Names, EventChatType,
                  EventMsgType, Msg, MessageEvent)
from .calendar import (EventReplyEvent, Attendee)
from .card import (CardAction, Action)
from .chat import (ChatDisbandEvent, UserChatEvent, GroupSettingUpdateEvent, GroupSetting)
from .contact import (UserAddEvent, DeptAddEvent, ContactScopeChangeEvent, UserStatusChangeEvent, UserStatus)



import asyncio

from feishu import FeishuClient

loop = asyncio.new_event_loop()
asyncio.set_event_loop(loop)
cli = FeishuClient(run_async=False)
cli_async = FeishuClient(run_async=True)


def test_tenant_access_token():
    token, expire = cli.get_tenant_access_token()
    assert token and expire > 0, "其实一般到不了这一步，要出错之前就出错了"
    assert token and expire

    token, expire = loop.run_until_complete(cli_async.get_tenant_access_token())
    assert token and expire
    print(f"token={token}, expire={expire}")


def test_bot_info():
    bot_info = cli.get_bot_info()
    assert bot_info
    bot_info = loop.run_until_complete(cli_async.get_bot_info())
    assert bot_info
    print(f"bot_info={bot_info}")


if __name__ == "__main__":
    test_tenant_access_token()
    test_bot_info()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""飞书通讯录事件

https://open.feishu.cn/document/ukTMukTMukTM/uITNxYjLyUTM24iM1EjN
"""
from .base import BaseModel, EventType, EventContent


class UserAddEvent(EventContent):
    """员工变更

    {
         "type": "user_add",    // 事件类型，包括user_add, user_update, user_leave
         "app_id": "cli_xxx",   // 应用ID
         "tenant_key": "xxx",   // 企业标识
         "open_id":"xxx" ,  // 员工对此应用的唯一标识，同一员工对不同应用的open_id不同
         "employee_id":"xxx",    // 即“用户ID”，仅企业自建应用会返回
         "union_id": "xxx" // 用户在ISV下的唯一标识，申请了"获取用户统一ID"权限后返回
    }
    """
    type: EventType = EventType.USER_ADD
    app_id: str
    tenant_key: str
    open_id: str
    employee_id: str = ''
    union_id: str = ''


class DeptAddEvent(EventContent):
    """部门变更

    {
         "type": "dept_add",  // 事件类型，包括 dept_add, dept_update, dept_delete
         "app_id": "cli_xxx",  // 应用ID
         "tenant_key": "xxx",           // 企业标识
         "open_department_id":"od-xxx"  // 部门的Id
     }
    """
    type: EventType = EventType.DEPT_ADD
    app_id: str
    tenant_key: str
    open_department_id: str


class UserStatus(BaseModel):
    is_active: bool
    is_frozen: bool
    is_resigned: bool


class UserStatusChangeEvent(EventContent):
    """用户状态变更

    {
        "type": "user_status_change",    // 事件类型
        "app_id": "cli_xxx",   // 应用ID
        "tenant_key": "xxx",   // 企业标识
        "open_id":"xxx" ,  // 员工对此应用的唯一标识，同一员工对不同应用的open_id不同
        "employee_id":"xxx",    // 即“用户ID”，仅企业自建应用会返回
        "union_id": "xxx",  //用户统一ID，申请了"获取用户统一ID"权限后返回
        "before_status": { // 变化前的状态
            "is_active": false,        // 账号是否已激活
            "is_frozen": false,       // 账号是否冻结
            "is_resigned": false    // 是否离职
        },
        "change_time": "2020-02-21 16:28:48", // 状态更新的时间
        "current_status": { // 变化后的状态
            "is_active": true,
            "is_frozen": false,
            "is_resigned": false
        }
    }
    """
    type: EventType = EventType.USER_STATUS_CHANGE
    app_id: str
    tenant_key: str = ''
    open_id: str = ''
    employee_id: str = ''
    union_id: str = ''
    before_status: UserStatus = {}
    change_time: str = ''
    current_status: UserStatus = {}


class ContactScopeChangeEvent(EventContent):
    """授权范围变更

    {
         "type": "contact_scope_change", // 事件类型
         "app_id": "cli_xxx",   // 应用ID
         "tenant_key": "xxx", //企业标识
     }
    """
    type: EventType = EventType.CONTACT_SCOPE_CHANGE
    app_id: str
    tenant_key: str



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""简易示例回调服务器"""
import asyncio
import logging
import os
import time
from concurrent.futures.thread import ThreadPoolExecutor

import requests

from .apis import setup_event_blueprint, setup_action_blueprint
from .consts import FEISHU_VERIFY_TOKEN, FEISHU_ENCRYPT_KEY
from .models import EventContent, CardAction

PATH_EVENT = "/feishu/callback/event"
PATH_ACTION = "/feishu/callback/action"


def create_simple_flask_server(verify_token: str = '', encrypt_key: str = '') -> "flask.Flask":
    """创建一个基于flask的简单的记录服务器

    记录：
        - 订阅事件(models.Event)
        - 互动消息(models.CardAction)
    """
    from flask import Flask, Blueprint
    app = Flask("feishu")

    def on_event(event: EventContent):
        app.logger.info(f"event: {event}")

    def on_action(action: CardAction):
        app.logger.info(f"action: {action}")

    event_app = Blueprint(name="event_app", import_name=__name__)
    setup_event_blueprint("flask", blueprint=event_app, path=PATH_EVENT,
                          on_event=on_event, verify_token=verify_token, encrypt_key=encrypt_key)

    action_app = Blueprint(name="action_app", import_name=__name__)
    setup_action_blueprint("flask", blueprint=action_app, path=PATH_ACTION,
                           on_action=on_action, verify_token=verify_token, encrypt_key=encrypt_key)

    app.register_blueprint(event_app)
    app.register_blueprint(action_app)

    app.logger.setLevel(logging.INFO)
    return app


def run_simple_flask_server(encrypt_key: str = '', verify_token: str = ''):
    if not encrypt_key:
        encrypt_key = os.environ.get(FEISHU_ENCRYPT_KEY, "").strip()
    if not verify_token:
        verify_token = os.environ.get(FEISHU_VERIFY_TOKEN, "").strip()
    app = create_simple_flask_server(encrypt_key=encrypt_key,
                                     verify_token=verify_token)
    port = 9000
    host = requests.get("http://api.ipify.org/").text

    def print_notice():
        time.sleep(0.5)
        app.logger.info("请确保服务器有公网静态IP!!!")
        app.logger.info(f"请在飞书后台配置订阅信息URL = 'http://{host}:{port}{PATH_EVENT}'")
        app.logger.info(f"请在飞书后台配置卡片互动消息URL = 'http://{host}:{port}{PATH_ACTION}'")

    executor = ThreadPoolExecutor(1)
    executor.submit(print_notice)

    app.run(host='0.0.0.0', port=port)


def create_simple_sanic_server(verify_token: str = '', encrypt_key: str = '') -> "sanic.Sanic":
    """创建一个基于sanic的简单的记录服务器

    记录：
        - 订阅事件(models.Event)
        - 互动消息(models.CardAction)
    """
    from sanic import Sanic, Blueprint
    from sanic.log import logger
    app = Sanic("feishu")

    async def on_event(event: EventContent):
        await asyncio.sleep(2)
        logger.info(f"event: {event}")

    async def on_action(action: CardAction):
        logger.info(f"action: {action}")

    event_app = Blueprint(name="event_app")
    setup_event_blueprint("sanic", blueprint=event_app, path=PATH_EVENT,
                          on_event=on_event, verify_token=verify_token, encrypt_key=encrypt_key)

    action_app = Blueprint(name="action_app")
    setup_action_blueprint("sanic", blueprint=action_app, path=PATH_ACTION,
                           on_action=on_action, verify_token=verify_token, encrypt_key=encrypt_key)

    app.blueprint(event_app)
    app.blueprint(action_app)

    logger.setLevel(logging.INFO)
    return app


def run_simple_sanic_server(encrypt_key: str = '', verify_token: str = ''):
    from sanic.log import logger
    if not encrypt_key:
        encrypt_key = os.environ.get(FEISHU_ENCRYPT_KEY, "").strip()
    if not verify_token:
        verify_token = os.environ.get(FEISHU_VERIFY_TOKEN, "").strip()
    app = create_simple_sanic_server(encrypt_key=encrypt_key,
                                     verify_token=verify_token)
    port = 9000
    host = requests.get("http://api.ipify.org/").text

    async def print_notice():
        logger.info("请确保服务器有公网静态IP!!!")
        logger.info(f"请在飞书后台配置订阅信息URL = 'http://{host}:{port}{PATH_EVENT}'")
        logger.info(f"请在飞书后台配置卡片互动消息URL = 'http://{host}:{port}{PATH_ACTION}'")

    app.add_task(print_notice())
    app.run(host='0.0.0.0', port=port)


if __name__ == '__main__':
    # run_simple_flask_server()
    run_simple_sanic_server()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""飞书事件类型声明

参考订阅事件文档 https://open.feishu.cn/document/ukTMukTMukTM/uUTNz4SN1MjL1UzM
"""
from enum import Enum
from typing import Union

from pydantic import BaseModel


class EventType(str, Enum):
    APP_OPEN = "app_open"
    APP_STATUS_CHANGE = "app_status_change"
    ORDER_PAID = "order_paid"
    APP_TICKET = "app_ticket"
    APP_UNINSTALLED = "app_uninstalled"

    USER_ADD = "user_add"
    DEPT_ADD = "dept_add"
    USER_STATUS_CHANGE = "user_status_change"
    CONTACT_SCOPE_CHANGE = "contact_scope_change"

    ADD_BOT = "add_bot"
    REMOVE_BOT = "remove_bot"
    P2P_CHAT_CREATE = "p2p_chat_create"
    MESSAGE = "message"
    MESSAGE_READ = "message_read"

    CHAT_DISBAND = "chat_disband"
    GROUP_SETTING_UPDATE = "group_setting_update"

    LEAVE_APPROVAL = "leave_approval"
    LEAVE_APPROVAL_V2 = "leave_approvalV2"
    WORK_APPROVAL = "work_approval"
    SHIFT_APPROVAL = "shift_approval"
    REMEDY_APPROVAL = "remedy_approval"
    TRIP_APPROVAL = "trip_approval"
    OUT_APPROVAL = "out_approval"

    EVENT_REPLY = "event_reply"

    ADD_USER_TO_CHAT = "add_user_to_chat"
    REMOVE_USER_FROM_CHAT = "remove_user_from_chat"
    REVOKE_ADD_USER_FROM_CHAT = "revoke_add_user_from_chat"


class AppStatus(str, Enum):
    START_BY_TENANT = "start_by_tenant"
    STOP_BY_TENANT = "stop_by_tenant"
    STOP_BY_PLATFORM = "stop_by_platform"


class BuyType(str, Enum):
    BUY = "buy"
    UPGRADE = "upgrade"
    RENEW = "renew"


class PricePlanType(str, Enum):
    PER_SEAT_PER_MONTH = "per_seat_per_month"


class UserChatEventType(str, Enum):
    ADD_USER_TO_CHAT = "add_user_to_chat"
    REMOVE_USER_FROM_CHAT = "remove_user_from_chat"
    REVOKE_ADD_USER_FROM_CHAT = "revoke_add_user_from_chat"


class EventContent(BaseModel):
    type: Union[str, EventType]
    user_agent: str = ''
    tenant_key: str = ''
    app_id: str = ''


class Event(BaseModel):
    """事件回调内容

    Args:
        ts 事件发送的时间，一般近似于事件发生的时间
        uuid 事件的唯一标识
        token 即verify_token, 可以和后台配置的token进行校验
        type "event_callback"
    """
    ts: str = ''
    uuid: str = ''
    token: str = ''
    type: str = "event_callback"
    event: Union[dict, EventContent]


class Operator(BaseModel):
    open_id: str
    user_id: str = ''


class User(BaseModel):
    name: str
    open_id: str
    user_id: str = ''



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""飞书应用事件

https://open.feishu.cn/document/ukTMukTMukTM/uQTNxYjL0UTM24CN1EjN
"""
from typing import List

from .base import BaseModel, EventType, AppStatus, PricePlanType, BuyType, EventContent


class Applicant(BaseModel):
    open_id: str
    user_id: str = ''


class Installer(BaseModel):
    open_id: str
    user_id: str = ''


class AppOpenEvent(EventContent):
    """首次开通应用

    {
        "app_id": "cli_xxx",  // 开通的应用ID
        "tenant_key":"xxx",   // 开通应用的企业唯一标识
        "type":"app_open",     // 事件类型
        "applicants": [ // 应用的申请者，可能有多个
            {
              "open_id":"xxx" ,  // 员工对此应用的唯一标识，同一员工对不同应用的open_id不同
              "user_id": "b78cfg77"//安装者的user_id （仅企业自建应用返回 ）
            }
        ],
        "installer": {	// 应用的安装者。如果是自动安装，则没有此字段。
        "open_id":"xxx" ,  // 员工对此应用的唯一标识，同一员工对不同应用的open_id不同
        "user_id": "b78cfg77"//安装者的user_id （仅企业自建应用返回 ）
    }
    """

    app_id: str
    tenant_key: str
    type: EventType = EventType.APP_OPEN
    applicants: List[Applicant]
    installer: Installer


class AppStatusChangeEvent(EventContent):
    """应用停启用

    {
        "app_id": "cli_xxx",  // 应用ID
        "tenant_key":"xxx",   // 企业唯一标识
        "type": "app_status_change",  // 事件类型
        "status": "start_by_tenant" //应用状态 start_by_tenant: 租户启用; stop_by_tenant: 租户停用; stop_by_platform: 平台停用
    }
    """
    app_id: str
    tenant_key: str
    type: EventType = EventType.APP_STATUS_CHANGE
    status: AppStatus


class OrderPaidEvent(EventContent):
    """应用商店应用购买

    {
        "type":"order_paid",     // 事件类型
        "app_id": "cli_9daeceab98721136", //应用ID
        "order_id": "6704894492631105539", // 用户购买付费方案时对订单ID 可作为唯一标识
        "price_plan_id": "price_9d86fa1333b8110c",  //付费方案ID
        "price_plan_type": "per_seat_per_month", // 用户购买方案类型
        "seats": 20, // 表示购买了多少人份
        "buy_count":1, //套餐购买数量 目前都为1
        "create_time": "1502199207",
        "pay_time": "1502199209",
        "buy_type": "buy", // 购买类型 buy普通购买 upgrade为升级购买 renew为续费购买
        "src_order_id": "6704894492631105539", // 当前为升级购买时(buy_type 为upgrade)，该字段表示原订单ID，升级后原订单失效，状态变为已升级(业务方需要处理)
        "order_pay_price":10000//订单支付价格 单位分，
        "tenant_key": "2f98c01bc23f6847"//购买应用的企业标示
    }
    """
    type: EventType = EventType.ORDER_PAID
    app_id: str
    order_id: str
    price_plan_id: str
    price_plan_type: str = PricePlanType.PER_SEAT_PER_MONTH
    seats: int
    buy_count: int
    create_time: str
    pay_time: str
    buy_type: BuyType
    src_order_id: str
    order_pay_price: int
    tenant_key: str


class AppTicketEvent(EventContent):
    """app_ticket事件

    对于应用商店应用，开放平台会每隔1小时推送一次 app_ticket ，应用通过该 app_ticket 获取 app_access_token。
    {
         "app_id": "cli_xxx",
         "app_ticket":"xxx",
         "type":"app_ticket"
    }
    """
    app_id: str
    app_ticket: str
    type: EventType = EventType.APP_TICKET


class AppUninstalledEvent(EventContent):
    """
    {
        "app_id": "cli_xxx", // 被卸载的应用ID
        "tenant_key": "xxx",  // 卸载应用的企业ID
        "type": "app_uninstalled"  // 事件类型
     }
    """
    app_id: str
    tenant_key: str
    type: EventType = EventType.APP_UNINSTALLED



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""飞书日历事件

https://open.feishu.cn/document/ukTMukTMukTM/uEzNxYjLxcTM24SM3EjN
"""

from .base import BaseModel, EventContent


class Attendee(BaseModel):
    open_id: str
    employee_id: str
    union_id: str


class EventReplyEvent(EventContent):
    """日程回复

    {
        "app_id" : "cli_xxxxxx",
        "type" : "event_reply", // 事件类型
        "tenant_key" : "xxxxx",
        "event_id": "xxxx", //日程id
        "attendee" : {
           "open_id":"xxx", // 回复此日程的参与人的open_id
           "employee_id":"yyy", // 回复此日程的参与人的employee_id，仅自建应用才会返回
           "union_id": "zzz" // 用户在ISV下的唯一标识，申请了"获取用户统一ID"权限后返回
        },
        "status": "accept", // 对日程的回复：accept 接受, tentative 待定, decline 拒绝
        "display_name":"xxx",
        "reply_timestmap":"1589097034" // 回复时间
    }
    """
    app_id: str
    type: str
    tenant_key: str
    event_id: str
    attendee: Attendee
    status: str
    display_name: str
    reply_timestamp: str



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""飞书审批事件

https://open.feishu.cn/document/ukTMukTMukTM/uQTNxYjL0UTM24CN1EjN
"""
from enum import Enum
from typing import List

from .base import BaseModel, EventType, EventContent


class TimeUnit(str, Enum):
    DAY = "DAY"
    HALF_DAY = "HALF_DAY"
    HOUR = "HOUR"


class LeaveApprovalEvent(EventContent):
    """请假审批

    {
         "app_id": "cli_xxx",
         "tenant_key":"xxx",
         "type": "leave_approval",
         "instance_code": "xxx", // 审批实例Code
         "employee_id": "xxx",  // 用户id
         "start_time": 1502199207, // 审批发起时间
         "end_time": 1502199307,   // 审批结束时间
         "leave_type": "xxx",      // 请假类型
         "leave_unit": 1,          // 请假最小时长：1：一天，2：半天
         "leave_start_time": "2018-12-01 12:00:00", // 请假开始时间
         "leave_end_time": "2018-12-02 12:00:00",   // 请假结束时间
         "leave_interval": 7200,          // 请假时长，单位（秒）
         "leave_reason": "xxx"     // 请假事由
    }
    """
    app_id: str
    tenant_key: str
    type: EventType = EventType.LEAVE_APPROVAL
    instance_code: str
    employee_id: str
    start_time: int
    end_time: int
    leave_type: str
    leave_unit: int
    leave_start_time: str
    leave_end_time: str
    leave_interval: int
    leave_reason: str


class I18nResource(BaseModel):
    """
    {
        "locale":"en_us",
        "is_default":true,
        "texts":{
            "@i18n@123456":"Holiday"
        }
    }
    """
    locale: str
    is_default: bool
    texts: dict


class LeaveApprovalV2Event(EventContent):
    """新版请假消息

    {
         "app_id": "cli_xxx",
         "tenant_key":"xxx",
         "type": "leave_approvalV2",
         "instance_code": "xxx", // 审批实例Code
         "user_id": "xxx",  // 用户id
         "start_time": 1564590532,  // 审批发起时间
         "end_time": 1564590532, // 审批结束时间
         "leave_name": "@i18n@123456",  // 假期名称
         "leave_unit": "DAY",  // 请假最小时长
         "leave_start_time": "2019-10-01 00:00:00",// 请假开始时间
         "leave_end_time":"2019-10-02 00:00:00",// 请假结束时间
         "leave_detail": [
            ["2019-10-01 00:00:00","2019-10-02 00:00:00"]
         ], // 具体请假明细
        "leave_interval": 86400,  // 请假时长，单位（秒）
        "leave_reason": "abc", // 请假事由
        "i18n_resources":[ {
                "locale":"en_us",
                "is_default":true,
                "texts":{
                    "@i18n@123456":"Holiday"
                }
        }  ] // 国际化文案
    }
    """
    app_id: str
    tenant_key: str
    type: EventType = EventType.LEAVE_APPROVAL_V2
    instance_code: str
    user_id: str
    start_time: int
    end_time: int
    leave_name: str
    leave_unit: TimeUnit
    leave_start_time: str
    leave_end_time: str
    leave_detail: List[List[str]]
    leave_interval: int
    leave_reason: str
    i18n_resources: List[I18nResource]


class WorkApprovalEvent(EventContent):
    """加班审批

    {
         "app_id": "cli_xxx",
         "tenant_key":"xxx",
         "type":"work_approval",
         "instance_code": "xxx", // 审批实例Code
         "employee_id": "xxx",  // 用户id
         "start_time": 1502199207, // 审批发起时间
         "end_time": 1502199307,   // 审批结束时间
         "work_type": "xxx",      // 加班类型
         "work_start_time": "2018-12-01 12:00:00", // 加班开始时间
         "work_end_time": "2018-12-02 12:00:00",   // 加班结束时间
         "work_interval": 7200,          // 加班时长，单位（秒）
         "work_reason": "xxx"     // 加班事由
    }
    """
    app_id: str
    tenant_key: str
    type: EventType = EventType.WORK_APPROVAL
    instance_code: str
    employee_id: str
    start_time: int
    end_time: int
    work_type: str
    work_start_time: str
    work_end_time: str
    work_interval: int
    work_reason: str


class ShiftApprovalEvent(EventContent):
    """换班审批

    {
         "app_id": "cli_xxx",
         "tenant_key":"xxx",
         "type":"shift_approval",
         "instance_code": "xxx", // 审批实例Code
         "employee_id": "xxx",  // 用户id
         "start_time": 1502199207, // 审批发起时间
         "end_time": 1502199307,   // 审批结束时间
         "shift_time": "2018-12-01 12:00:00",    // 换班时间
         "return_time": "2018-12-02 12:00:00",   // 还班时间
         "shift_reason": "xxx"     // 换班事由
    }
    """
    app_id: str
    tenant_key: str
    type: EventType = EventType.REMEDY_APPROVAL
    instance_code = str
    employee_id: str
    start_time: int
    end_time: int
    shift_time: str
    return_time: str
    shift_reason: str


class RemedyApprovalEvent(EventContent):
    """补卡审批

    {
         "app_id": "cli_xxx",
         "tenant_key":"xxx",
         "type":"remedy_approval",
         "instance_code": "xxx", // 审批实例Code
         "employee_id": "xxx",  // 用户id
         "start_time": 1502199207, // 审批发起时间
         "end_time": 1502199307,   // 审批结束时间
         "remedy_time": "2018-12-01 12:00:00",   // 补卡时间
         "remedy_reason": "xxx"    // 补卡原因
    }
    """
    app_id: str
    tenant_key: str
    type: EventType = EventType.REMEDY_APPROVAL
    instance_code = str
    employee_id: str
    start_time: int
    end_time: int
    remedy_time: str
    remedy_reason: str


class Schedule(BaseModel):
    """
    {         // Schedule 结构数组
        "trip_start_time": "2018-12-01 12:00:00", // 行程开始时间
        "trip_end_time":  "2018-12-01 12:00:00", // 行程结束时间
        "trip_interval":  3600, // 行程时长（秒）
        "departure":  "xxx",       // 出发地
        "destination":  "xxx",       // 目的地
        "transportation":  "xxx",       // 目的地
        "trip_type": "单程", // 单程/往返
        "remark": "备注"    // 备注
    }
    """
    trip_start_time: str
    trip_end_time: str
    trip_interval: int
    departure: str
    destination: str
    transportation: str
    trip_type: str
    remark: str


class TripApprovalEvent(EventContent):
    """出差审批

    {
         "app_id": "cli_xxx",
         "tenant_key":"xxx",
         "type":"trip_approval",
         "instance_code": "xxx", // 审批实例Code
         "employee_id": "xxx",  // 用户id
         "start_time": 1502199207, // 审批发起时间
         "end_time": 1502199307,   // 审批结束时间
         "schedules": [{         // Schedule 结构数组
                "trip_start_time": "2018-12-01 12:00:00", // 行程开始时间
                "trip_end_time":  "2018-12-01 12:00:00", // 行程结束时间
                "trip_interval":  3600, // 行程时长（秒）
                "departure":  "xxx",       // 出发地
                "destination":  "xxx",       // 目的地
                "transportation":  "xxx",       // 目的地
                "trip_type": "单程", // 单程/往返
                "remark": "备注"    // 备注
                },
         ]
         "trip_interval": 3600,    // 行程总时长（秒）
         "trip_reason": "xxx"     // 出差事由
         "trip_peers": ["xxx", "yyy"],   // 同行人
    }
    """
    app_id: str
    tenant_key: str
    type: str
    instance_code: str
    employee_id: str
    start_time: int
    end_time: int
    schedules: List[Schedule]
    trip_interval: int
    trip_reason: str
    trip_peers: List[str]


class OutApprovalEvent(EventContent):
    """外出审批事件

    {
        'app_id': 'cli_9e28cb7ba56a100e',
        'i18n_resources': [{
          'is_default': True,
          'locale': 'zh_cn',
          'texts': {
            '@i18n@someKey': '中文文案' // key对应的文案
          }
        }],
        'instance_code': '59558CEE-CEF4-45C9-A2C3-DCBF8BEC7341', // 此审批的唯一标识
        'out_image': '',
        'out_interval': 10800, // 外出时长，单位秒
        'out_name': '@i18n@someKey', // 通过i18n_resources里的信息换取相应语言的文案
        'out_reason': '外出事由',
        'out_start_time': '2020-05-15 15:00:00',
        'out_end_time': '2020-05-15 18:00:00',
        'out_unit': 'HOUR', // 外出时长的单位，HOUR 小时，DAY 天，HALF_DAY 半天
        'start_time': 1589527346, // 审批开始时间
        'end_time': 1589527354, // 审批结束时间
        'tenant_key': '2d520d3b434f175e', // 企业唯一标识
        'type': 'out_approval', // 事件类型
        'user_id': 'g6964gd3' // 申请发起人
    }
    """
    app_id: str
    i18n_resources: List[I18nResource]
    instance_code: str
    out_image: str
    out_interval: int
    out_name: str
    out_reason: str
    out_start_time: str
    out_end_time: str
    out_unit: TimeUnit
    start_time: int
    end_time: int
    tenant_key: str
    type: str
    user_id: str



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""飞书卡片消息事件

https://open.feishu.cn/document/ukTMukTMukTM/uYjNwUjL2YDM14iN2ATN

卡片消息事件和其他事件不一样，不是继承自base.Event
最明显的区别是
- event_callback事件外层有ts/uuid/token等字段，卡片消息没有
- event_callback中的token是校验token, 卡片消息中的token是用来刷新消息卡片内容的
"""
from typing import Optional

from .base import BaseModel


class Action(BaseModel):
    """交互信息

    Args:
        value 交互元素的value字段, key: value对
        tag 交互元素的tag值
        option 选中option的value(button元素不适用)
    """

    value: Optional[dict] = None
    tag: str
    option: str = ""


class CardAction(BaseModel):
    """卡片互动事件

    https://open.feishu.cn/document/ukTMukTMukTM/uYzMxEjL2MTMx4iNzETM

    Args:
        refresh_token 请求去重,
            需要特别提一下，这个并不是请求体里面的，而是headers['X-Refresh-Token']
            业务方通过验证 headers['X-Refresh-Token'] 来防止按钮事件被重复处理。
            在网络抖动等极端情况下，会出现卡片点击失败但是业务方已经处理过 action 的现象，
            所以业务方接口存在被重复调用的风险。
            X-Refresh-Token 只有在卡片点击事件成功被响应并通知到客户端的时候才会刷新，
            如果业务方的接口非幂等，可以通过缓存并验证该字段防止接口被重复调用。
    """

    open_id: str
    user_id: str
    tenant_key: str
    open_message_id: str
    token: str
    action: Action
    refresh_token: Optional[str]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from enum import Enum

FEISHU_TOKEN_EXPIRE_TIME = 7200  # token时效, https://open.feishu.cn/document/ukTMukTMukTM/uIjNz4iM2MjLyYzM
FEISHU_TOKEN_UPDATE_TIME = 600  # token提前更新的时间
FEISHU_BATCH_SEND_SIZE = 200  # 批量发送消息列表的大小限制

# 环境变量名
FEISHU_APP_ID = "FEISHU_APP_ID"
FEISHU_APP_SECRET = "FEISHU_APP_SECRET"
FEISHU_VERIFY_TOKEN = "FEISHU_VERIFY_TOKEN"
FEISHU_ENCRYPT_KEY = "FEISHU_ENCRYPT_KEY"


class AppType(str, Enum):
    TENANT = "tenant"  # 企业自建应用
    USER = "user"  # 第三方应用



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""飞书群聊事件

https://open.feishu.cn/document/ukTMukTMukTM/ukjNxYjL5YTM24SO2EjN
"""
from typing import List

from .base import BaseModel, EventType, UserChatEventType, Operator, User, EventContent


class UserChatEvent(EventContent):
    """用户进群、出群事件

    特殊说明：只有开启机器人能力并且机器人所在的群发生上述变化时才能触发此事件。
    {
        "app_id": "cli_9c8609450f78d102",
        "chat_id": "oc_9e9619b938c9571c1c3165681cdaead5", // 群聊的id
        "operator": {
          // 用户进出群的操作人。用户主动退群的话，operator 就是user自己
          "open_id": "ou_18eac85d35a26f989317ad4f02e8bbbb", // 员工对此应用的唯一标识，同一员工对不同应用的open_id不同
          "user_id": "ca51d83b" // 即“用户ID”，仅企业自建应用会返回
        },
        "tenant_key": "736588c9260f175d",
        "type": "add_user_to_chat", // 事件类型，add_user_to_chat/remove_user_from_chat/revoke_add_user_from_chat
        "users": [{
            "name": "James",
            "open_id": "ou_706adeb944ab1473b9fb3e7da2a40b68",
            "user_id": "51g97a4g"
          },
          {
            "name": "张三",
            "open_id": "ou_7885357f9922aaa34001b190109e0b48",
            "user_id": "6e125386"
          }
        ]
    }
    """
    app_id: str
    chat_id: str
    operator: Operator
    tenant_key: str
    type: UserChatEventType
    users: List[User]


class ChatDisbandEvent(EventContent):
    """解散群

    特殊说明：只有开启机器人能力并且机器人所在的群被解散时才能触发此事件。
    {
        "app_id": "cli_9c8609450f78d102",
        "chat_id": "oc_9f2df3c095c9395334bb6e70ced0fa83",
        "operator": {
            "open_id": "ou_18eac85d35a26f989317ad4f02e8bbbb",
            "user_id": "ca51d83b"
        },
        "tenant_key": "736588c9260f175d",
        "type": "chat_disband"
    }
    """
    app_id: str
    chat_id: str
    operator: Operator
    tenant_key: str
    type: str


class GroupSetting(BaseModel):
    """
    {
          // 当群主发生变化时会有下面2个字段。若群主未发生变化，则不会有这2个字段。
          'owner_open_id': 'ou_b5f49f047428e47925599be05e3255f6',
          'owner_user_id': '35g25a59',
          // 当“仅群主可添加群成员”配置变化时有下面的字段。
          'add_member_permission': 'everyone',
          // 当“消息提醒”配置变化时有下面的字段。
          'message_notification': False
    }
    """
    owner_open_id: str
    owner_user_id: str = ''
    add_member_permission: str
    message_notification: bool


class GroupSettingUpdateEvent(EventContent):
    """群配置更改

    特殊说明：只有开启机器人能力并且机器人所在的群发生上述变化时才能触发此事件。
    {
        'tenant_key': '2d520d3b434f175e',
        'type': 'group_setting_update'
        'app_id': 'cli_9e28cb7ba56a100e',
        'chat_id': 'oc_066cad06159f0752fe02c9af8aebfc5a',
        'after_change': {
          // 当群主发生变化时会有下面2个字段。若群主未发生变化，则不会有这2个字段。
          'owner_open_id': 'ou_b5f49f047428e47925599be05e3255f6',
          'owner_user_id': '35g25a59',
          // 当“仅群主可添加群成员”配置变化时有下面的字段。
          'add_member_permission': 'everyone',
          // 当“消息提醒”配置变化时有下面的字段。
          'message_notification': False
        },
        'before_change': {
          // after_change里有几个字段，这里就有几个与之对应。
        },
        'operator': { // 配置变化的操作者
          'open_id': 'ou_55a50f4c6eba6adbfc8b94803fe78825',
          'user_id': 'g6964gd3'
        }
    }
    """
    tenant_key: str
    type: EventType = EventType.GROUP_SETTING_UPDATE
    app_id: str
    chat_id: str
    after_change: GroupSetting
    before_change: GroupSetting
    operator: Operator



"""Prepare a Rasa OSS release.
- creates a release branch
- creates a new changelog section in CHANGELOG.mdx based on all collected changes
- increases the version number
- pushes the new branch to GitHub
"""
import argparse
import os
import re
import sys
from pathlib import Path
from subprocess import CalledProcessError, check_call, check_output
from typing import Text, Set

import questionary
from pep440_version_utils import Version, is_valid_version

VERSION_FILE_PATH = "feishu/version.py"

RELEASE_BRANCH_PREFIX = "prepare-release-"

PRERELEASE_FLAVORS = ("alpha", "rc")

RELEASE_BRANCH_PATTERN = re.compile(r"^\d+\.\d+\.x$")

REPO_BASE_URL = "https://github.com/cyclone-robotics/feishu-python-sdk"


def create_argument_parser() -> argparse.ArgumentParser:
    """Parse all the command line arguments for the release script."""

    parser = argparse.ArgumentParser(description="prepare the next library release")
    parser.add_argument(
        "--next_version",
        type=str,
        help="Either next version number or 'major', 'minor', 'micro', 'alpha', 'rc'",
    )

    return parser


def project_root() -> Path:
    """Root directory of the project."""
    return Path(os.path.dirname(__file__)).parent


def version_file_path() -> Path:
    """Path to the python file containing the version number."""
    return project_root() / VERSION_FILE_PATH


def write_version_file(version: Version) -> None:
    """Dump a new version into the python version file."""

    with version_file_path().open("w") as f:
        f.write(
            '# 不要修改此文件，本文件由release.py自动生成\n'
            f'__version__ = "{version}"\n'
        )
    check_call(["git", "add", str(version_file_path().absolute())])


def get_current_version() -> Text:
    """当前版本号"""

    if not version_file_path().is_file():
        raise FileNotFoundError(
            f"Failed to find version file at {version_file_path().absolute()}"
        )

    # context in which we evaluate the version py -
    # to be able to access the defined version, it already needs to live in the
    # context passed to exec
    _globals = {"__version__": ""}
    with version_file_path().open() as f:
        exec(f.read(), _globals)

    return _globals["__version__"]


def confirm_version(version: Version) -> bool:
    """确认是否覆盖版本号"""

    if str(version) in git_existing_tags():
        confirmed = questionary.confirm(
            f"Tag with version '{version}' already exists, overwrite?", default=False
        ).ask()
    else:
        confirmed = questionary.confirm(
            f"Current version is '{get_current_version()}. "
            f"Is the next version '{version}' correct ?",
            default=True,
        ).ask()
    if confirmed:
        return True
    else:
        print("Aborting.")
        sys.exit(1)


def ask_version() -> Text:
    """询问新版本号"""

    def is_valid_version_number(v: Text) -> bool:
        return v in {"major", "minor", "micro", "alpha", "rc"} or is_valid_version(v)

    current_version = Version(get_current_version())
    next_micro_version = str(current_version.next_micro())
    next_alpha_version = str(current_version.next_alpha())
    version = questionary.text(
        f"What is the version number you want to release "
        f"('major', 'minor', 'micro', 'alpha', 'rc' or valid version number "
        f"e.g. '{next_micro_version}' or '{next_alpha_version}')?",
        validate=is_valid_version_number,
    ).ask()

    if version in PRERELEASE_FLAVORS and not current_version.pre:
        # at this stage it's hard to guess the kind of version bump the
        # releaser wants, so we ask them
        if version == "alpha":
            choices = [
                str(current_version.next_alpha("minor")),
                str(current_version.next_alpha("micro")),
                str(current_version.next_alpha("major")),
            ]
        else:
            choices = [
                str(current_version.next_release_candidate("minor")),
                str(current_version.next_release_candidate("micro")),
                str(current_version.next_release_candidate("major")),
            ]
        version = questionary.select(
            f"Which {version} do you want to release?", choices=choices,
        ).ask()

    if version:
        return version
    else:
        print("Aborting.")
        sys.exit(1)


def validate_code_is_release_ready(version: Version) -> None:
    """Make sure the code base is valid (e.g. Rasa SDK is up to date)."""
    return True


def git_existing_tags() -> Set[Text]:
    """Return all existing tags in the local git repo."""

    stdout = check_output(["git", "tag"])
    return set(stdout.decode().split("\n"))


def git_current_branch() -> Text:
    """Returns the current git branch of the local repo."""

    try:
        output = check_output(["git", "symbolic-ref", "--short", "HEAD"])
        return output.decode().strip()
    except CalledProcessError:
        # e.g. we are in detached head state
        return "master"


def git_current_branch_is_master_or_release() -> bool:
    """
    Returns True if the current local git
    branch is master or a release branch e.g. 1.10.x
    """
    current_branch = git_current_branch()
    return (
            current_branch == "master"
            or RELEASE_BRANCH_PATTERN.match(current_branch) is not None
    )


def create_release_branch(version: Version) -> Text:
    """Create a new branch for this release. Returns the branch name."""

    branch = f"{RELEASE_BRANCH_PREFIX}{version}"
    check_call(["git", "checkout", "-b", branch])
    return branch


def create_commit(version: Version) -> None:
    """Creates a git commit with all stashed changes."""
    check_call(["git", "commit", "-m", f"prepared release of version {version}"])


def push_changes() -> None:
    """Pushes the current branch to origin."""
    check_call(["git", "push", "origin", "HEAD"])


def ensure_clean_git() -> None:
    """Makes sure the current working git copy is clean."""

    try:
        check_call(["git", "diff-index", "--quiet", "HEAD", "--"])
    except CalledProcessError:
        print("Your git is not clean. Release script can only be run from a clean git.")
        sys.exit(1)


def parse_next_version(version: Text) -> Version:
    """Find the next version as a proper semantic version string."""
    if version == "major":
        return Version(get_current_version()).next_major()
    elif version == "minor":
        return Version(get_current_version()).next_minor()
    elif version == "micro":
        return Version(get_current_version()).next_micro()
    elif version == "alpha":
        return Version(get_current_version()).next_alpha()
    elif version == "rc":
        return Version(get_current_version()).next_release_candidate()
    elif is_valid_version(version):
        return Version(version)
    else:
        raise Exception(f"Invalid version number '{cmdline_args.next_version}'.")


def next_version(args: argparse.Namespace) -> Version:
    """Take cmdline args or ask the user for the next version and return semver."""
    return parse_next_version(args.next_version or ask_version())


def generate_changelog(version: Version) -> None:
    """Call tonwcrier and create a changelog from all available changelog entries."""
    # check_call(
    #     ["towncrier", "--yes", "--version", str(version)], cwd=str(project_root())
    # )
    return


def print_done_message(branch: Text, base: Text, version: Version) -> None:
    """Print final information for the user on what to do next."""

    pull_request_url = f"{REPO_BASE_URL}/compare/{base}...{branch}?expand=1"

    print()
    print(f"\033[94m All done - changes for version {version} are ready! \033[0m")
    print()
    print(f"Please open a PR on GitHub: {pull_request_url}")


def print_done_message_same_branch(version: Version) -> None:
    """
    Print final information for the user in case changes
    are directly committed on this branch.
    """

    print()
    print(
        f"\033[94m All done - changes for version {version} where committed on this branch \033[0m"
    )


def main(args: argparse.Namespace) -> None:
    """Start a release preparation."""

    print(
        "本脚本将提升version版本，创建changelog并发布一个release branch",
    )

    ensure_clean_git()
    version = next_version(args)
    confirm_version(version)

    validate_code_is_release_ready(version)

    write_version_file(version)

    if not version.pre:
        # never update changelog on a prerelease version
        generate_changelog(version)

    # alpha workflow on feature branch when a version bump is required
    if version.is_alpha and not git_current_branch_is_master_or_release():
        create_commit(version)
        push_changes()

        print_done_message_same_branch(version)
    else:
        base = git_current_branch()
        branch = create_release_branch(version)

        create_commit(version)
        push_changes()

        print_done_message(branch, base, version)


if __name__ == "__main__":
    arg_parser = create_argument_parser()
    cmdline_args = arg_parser.parse_args()
    main(cmdline_args)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from feishu.apis.base import decrypt_aes


def test_decrypt_aes():
    encrypted = "FIAfJPGRmFZWkaxPQ1XrJZVbv2JwdjfLk4jx0k/U1deAqYK3AXOZ5zcHt/cC4ZNTqYwWUW/EoL+b2hW/C4zoAQQ5CeMtbxX2zHjm+E4nX/Aww+FHUL6iuIMaeL2KLxqdtbHRC50vgC2YI7xohnb3KuCNBMUzLiPeNIpVdnYaeteCmSaESb+AZpJB9PExzTpRDzCRv+T6o5vlzaE8UgIneC1sYu85BnPBEMTSuj1ZZzfdQi7ZW992Z4dmJxn9e8FL2VArNm99f5Io3c2O4AcNsQENNKtfAAxVjCqc3mg5jF0nKabA+u/5vrUD76flX1UOF5fzJ0sApG2OEn9wfyPDRBsApn9o+fceF9hNrYBGsdtZrZYyGG387CGOtKsuj8e2E8SNp+Pn4E9oYejOTR+ZNLNi+twxaXVlJhr6l+RXYwEiMGQE9zGFBD6h2dOhKh3W84p1GEYnSRIz1+9/Hp66arjC7RCrhuW5OjCj4QFEQJiwgL45XryxHtiZ7JdAlPmjVsL03CxxFZarzxzffryrWUG3VkRdHRHbTsC34+ScoL5MTDU1QAWdqUC1T7xT0lCvQELaIhBTXAYrznJl6PlA83oqlMxpHh0gZBB1jFbfoUr7OQbBs1xqzpYK6Yjux6diwpQB1zlZErYJUfCqK7G/zI9yK/60b4HW0k3M+AvzMcw="
    result = decrypt_aes("kudryavka", encrypted)
    assert result == {'uuid': '5226cd85b4d843dccee2e279d93f3ed3',
                      'event': {'app_id': 'cli_9e28cb7ba56a100e',
                                'before_status': {'is_active': True,
                                                  'is_frozen': True,
                                                  'is_resigned': False},
                                'change_time': '2020-05-20 18:33:25',
                                'current_status': {'is_active': True,
                                                   'is_frozen': False,
                                                   'is_resigned': False},
                                'employee_id': '75ge6c49', 'open_id':
                                    'ou_2ef04637d933f798dcb92c99e845ed09',
                                'tenant_key': '2d520d3b434f175e',
                                'type': 'user_status_change'},
                      'token': 'GzhQEyfUcx7eEungQFWtXgCbxSpUOJIb',
                      'ts': '1589970805.376395',
                      'type': 'event_callback'}


if __name__ == "__main__":
    test_decrypt_aes()



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="feishu-python-sdk",
    version="0.1.3",
    author="Cyclone Chatbot Dev",
    author_email="jingchao.hu@cyclone-robotics.com",
    description="一个用于和飞书开放平台交互的python库",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/cyclone-robotics/feishu-python-sdk",
    install_requires=[
        "pydantic>=1.6.1",
        "Flask>=1.1.2",
        "aiohttp>=3.7.3",
        "requests>=2.24.0",
        "cryptography>=3.1",
    ],
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
)



import asyncio
from asyncio import Future
from typing import Union

from feishu.apis.base import allow_async_call


class TheClient:
    run_async = True
    event_loop = asyncio.get_event_loop()

    def request(self, arg: str) -> Union[str, Future]:
        if self.run_async:
            async def _async():
                return arg

            return asyncio.ensure_future(_async(), loop=self.event_loop)
        else:
            return arg


class TheAPI:
    client = TheClient()

    @allow_async_call
    def the_method(self, arg1: str) -> str:
        arg2 = arg1 + "2"
        result = self.client.request(arg2)
        return result


def test_sync_call():
    api = TheAPI()
    api.client.run_async = False
    assert "sync2" == api.the_method("sync")
    print("sync2")


def test_async_call():
    api = TheAPI()
    api.client.run_async = True
    future: asyncio.Future = api.the_method("async")
    api.client.event_loop.run_until_complete(future)
    assert "async2" == future.result()
    print("async2")


if __name__ == "__main__":
    test_async_call()
    test_sync_call()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .api import FeishuAPI
from .base import _get_or_create_event_loop, allow_async_call
from .card import setup_action_blueprint
from .event import setup_event_blueprint, guess_event



# /usr/bin/env python
# -*- coding: utf-8 -*-
from typing import Tuple

from .base import BaseAPI, allow_async_call


class AuthAPI(BaseAPI):
    @allow_async_call
    def get_tenant_access_token(self) -> Tuple[str, int]:
        """获取自建应用的token

        https://open.feishu.cn/document/ukTMukTMukTM/uIjNz4iM2MjLyYzM

        Returns:
            Tuple[token, expire]
        """
        api = "/auth/v3/tenant_access_token/internal/"
        payload = dict(
            app_id=self.client.app_id,
            app_secret=self.client.app_secret,
        )
        result = self.client.request("POST", api=api, payload=payload, auth=False)
        # 返回 Body ：
        # {
        #     "code":0,
        #     "msg":"ok",
        #     "tenant_access_token":"xxxxx",
        #     "expire":7200  // 过期时间，单位为秒（两小时失效）
        # }
        return result["tenant_access_token"], result["expire"]

    def resend_app_ticket(self):
        """重新推送app_ticket

        https://open.feishu.cn/document/ukTMukTMukTM/uQjNz4CN2MjL0YzM
        """
        NotImplementedError
